@@ -19,8 +19,8 @@ DEFAULTS: ConfigDict = {
     "SG_ENGINE_DB_NAME": "splitgraph",
     "SG_ENGINE_USER": "sgr",
     "SG_ENGINE_PWD": "supersecure",
-    "SG_ENGINE_ADMIN_USER": "sgr",
-    "SG_ENGINE_ADMIN_PWD": "supersecure",
+    "SG_ENGINE_ADMIN_USER": "",
+    "SG_ENGINE_ADMIN_PWD": "",
     "SG_ENGINE_POSTGRES_DB_NAME": "postgres",
     "SG_ENGINE_OBJECT_PATH": "/var/lib/splitgraph/objects",
     # Multicorn/LQFDW currently doesn't give good estimates for GroupBy
@@ -138,8 +138,8 @@ This can also be changed by passing `--verbosity` to `sgr`, e.g. `sgr --verbosit
     "SG_ENGINE_DB_NAME": "Database used by Splitgraph.",
     "SG_ENGINE_USER": "Username used by sgr.",
     "SG_ENGINE_PWD": "Password used by sgr.",
-    "SG_ENGINE_ADMIN_USER": "Superuser username for the engine, used to first initialize it and create the required Splitgraph schemata and extensions.",
-    "SG_ENGINE_ADMIN_PWD": "Superuser password for the engine, used to first initialize it and create the required Splitgraph schemata and extensions.",
+    "SG_ENGINE_ADMIN_USER": "Superuser username for the engine, used to first initialize it and create the required Splitgraph schemata and extensions. The one user created on the standard engine is also a superuser, so you only need to override this if you created more than one user on your engine.",
+    "SG_ENGINE_ADMIN_PWD": "Superuser password for the engine, used to first initialize it and create the required Splitgraph schemata and extensions. Defaults to the same as SG_ENGINE_PWD.",
     "SG_ENGINE_POSTGRES_DB_NAME": "Name of the default database that the superuser connects to to initialize Splitgraph.",
     "SG_ENGINE_OBJECT_PATH": "Path on the engine's filesystem where Splitgraph physical object files are stored.",
     "SG_LQ_TUNING": "Postgres query planner configuration for Splitfile execution and table imports. This is run before a layered query is executed and allows to tune query planning in case of LQ performance issues. For possible values, see the [PostgreSQL documentation](https://www.postgresql.org/docs/12/runtime-config-query.html).",
