@@ -1,17 +1,25 @@
+import json
 import logging
+import re
 from abc import ABC, abstractmethod
 from copy import deepcopy
-from typing import Optional, Mapping, Dict, List, Any, cast, Union, TYPE_CHECKING
+from typing import Optional, Mapping, Dict, List, Any, cast, TYPE_CHECKING, Tuple
 
 import psycopg2
 from psycopg2.sql import SQL, Identifier
 
-from splitgraph.core.types import dict_to_tableschema, TableSchema, TableColumn
-from splitgraph.hooks.data_source.base import (
-    Credentials,
-    Params,
+from splitgraph.core.types import (
+    TableSchema,
+    TableColumn,
+    dict_to_table_schema_params,
+    TableParams,
     TableInfo,
     PreviewResult,
+    MountError,
+    IntrospectionResult,
+)
+from splitgraph.exceptions import get_exception_name
+from splitgraph.hooks.data_source.base import (
     MountableDataSource,
     LoadableDataSource,
 )
@@ -20,24 +28,17 @@ if TYPE_CHECKING:
     from splitgraph.engine.postgres.engine import PostgresEngine
 
 
-_table_options_schema = {
-    "type": "object",
-    "additionalProperties": {
-        "options": {"type": "object", "additionalProperties": {"type": "string"}},
-    },
-}
-
-
 class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC):
-    credentials_schema = {
+    credentials_schema: Dict[str, Any] = {
         "type": "object",
-        "properties": {"username": {"type": "string"}, "password": {"type": "string"}},
-        "required": ["username", "password"],
     }
 
-    params_schema = {
+    params_schema: Dict[str, Any] = {
+        "type": "object",
+    }
+
+    table_params_schema: Dict[str, Any] = {
         "type": "object",
-        "properties": {"tables": _table_options_schema},
     }
 
     commandline_help: str = ""
@@ -46,15 +47,6 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
     supports_mount = True
     supports_load = True
 
-    def __init__(
-        self,
-        engine: "PostgresEngine",
-        credentials: Optional[Credentials] = None,
-        params: Optional[Params] = None,
-    ):
-        self.tables: Optional[TableInfo] = None
-        super().__init__(engine, credentials or {}, params or {})
-
     @classmethod
     def from_commandline(cls, engine, commandline_kwargs) -> "ForeignDataWrapperDataSource":
         """Instantiate an FDW data source from commandline arguments."""
@@ -64,28 +56,19 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
 
         # By convention, the "tables" object can be:
         #   * A list of tables to import
-        #   * A dictionary table_name -> {"schema": schema, **mount_options}
-        table_kwargs = params.pop("tables", None)
-        tables: Optional[TableInfo]
-
-        if isinstance(table_kwargs, dict):
-            tables = dict_to_tableschema(
-                {t: to["schema"] for t, to in table_kwargs.items() if "schema" in to}
-            )
-            params["tables"] = {
-                t: {"options": to["options"]} for t, to in table_kwargs.items() if "options" in to
-            }
-        elif isinstance(table_kwargs, list):
-            tables = table_kwargs
-        else:
-            tables = None
+        #   * A dictionary table_name -> {"schema": schema, "options": table options}
+        tables = params.pop("tables", None)
+        if isinstance(tables, dict):
+            tables = dict_to_table_schema_params(tables)
 
+        # Extract credentials from the cmdline params
         credentials: Dict[str, Any] = {}
         for k in cast(Dict[str, Any], cls.credentials_schema["properties"]).keys():
             if k in params:
                 credentials[k] = params[k]
-        result = cls(engine, credentials, params)
-        result.tables = tables
+
+        result = cls(engine, credentials, params, tables)
+
         return result
 
     def get_user_options(self) -> Mapping[str, str]:
@@ -94,10 +77,19 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
     def get_server_options(self) -> Mapping[str, str]:
         return {}
 
-    def get_table_options(self, table_name: str) -> Mapping[str, str]:
+    def get_table_options(
+        self, table_name: str, tables: Optional[TableInfo] = None
+    ) -> Dict[str, str]:
+        tables = tables or self.tables
+
+        if not isinstance(tables, dict):
+            return {}
+
         return {
             k: str(v)
-            for k, v in self.params.get("tables", {}).get(table_name, {}).get("options", {}).items()
+            for k, v in tables.get(table_name, cast(Tuple[TableSchema, TableParams], ({}, {})))[
+                1
+            ].items()
         }
 
     def get_table_schema(self, table_name: str, table_schema: TableSchema) -> TableSchema:
@@ -113,8 +105,11 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
         pass
 
     def mount(
-        self, schema: str, tables: Optional[TableInfo] = None, overwrite: bool = True,
-    ):
+        self,
+        schema: str,
+        tables: Optional[TableInfo] = None,
+        overwrite: bool = True,
+    ) -> Optional[List[MountError]]:
         tables = tables or self.tables or []
 
         fdw = self.get_fdw_name()
@@ -131,9 +126,15 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
         # Allow mounting tables into existing schemas
         self.engine.run_sql(SQL("CREATE SCHEMA IF NOT EXISTS {}").format(Identifier(schema)))
 
-        self._create_foreign_tables(schema, server_id, tables)
+        errors = self._create_foreign_tables(schema, server_id, tables)
+        if errors:
+            for e in errors:
+                logging.warning("Error mounting %s: %s: %s", e.table_name, e.error, e.error_text)
+        return errors
 
-    def _create_foreign_tables(self, schema, server_id, tables):
+    def _create_foreign_tables(
+        self, schema: str, server_id: str, tables: TableInfo
+    ) -> List[MountError]:
         if isinstance(tables, list):
             try:
                 remote_schema = self.get_remote_schema_name()
@@ -141,33 +142,50 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
                 raise NotImplementedError(
                     "The FDW does not support IMPORT FOREIGN SCHEMA! Pass a tables dictionary."
                 )
-            _import_foreign_schema(self.engine, schema, remote_schema, server_id, tables)
+            return import_foreign_schema(self.engine, schema, remote_schema, server_id, tables)
         else:
-            for table_name, table_schema in tables.items():
+            for table_name, (table_schema, _) in tables.items():
                 logging.info("Mounting table %s", table_name)
                 query, args = create_foreign_table(
                     schema,
                     server_id,
                     table_name,
                     schema_spec=self.get_table_schema(table_name, table_schema),
-                    extra_options=self.get_table_options(table_name),
+                    extra_options=self.get_table_options(table_name, tables),
                 )
                 self.engine.run_sql(query, args)
+            return []
 
-    def introspect(self) -> Dict[str, TableSchema]:
+    def introspect(self) -> IntrospectionResult:
         # Ability to override introspection by e.g. contacting the remote database without having
         # to mount the actual table. By default, just call out into mount()
 
         # Local import here since this data source gets imported by the commandline entry point
         from splitgraph.core.common import get_temporary_table_id
+        import jsonschema
 
         tmp_schema = get_temporary_table_id()
         try:
-            self.mount(tmp_schema)
-            result = {
-                t: self.engine.get_full_table_schema(tmp_schema, t)
+            mount_errors = self.mount(tmp_schema) or []
+
+            table_options = dict(self._get_foreign_table_options(tmp_schema))
+
+            # Sanity check for adapters: validate the foreign table options that we get back
+            # to make sure they're still appropriate.
+            for v in table_options.values():
+                jsonschema.validate(v, self.table_params_schema)
+
+            result: IntrospectionResult = {
+                t: (
+                    self.engine.get_full_table_schema(tmp_schema, t),
+                    cast(TableParams, table_options.get(t, {})),
+                )
                 for t in self.engine.get_all_tables(tmp_schema)
             }
+
+            # Add errors to the result as well
+            for error in mount_errors:
+                result[error.table_name] = error
             return result
         finally:
             self.engine.rollback()
@@ -189,7 +207,36 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
         )
         return result_json
 
-    def preview(self, schema: Dict[str, TableSchema]) -> PreviewResult:
+    def _get_foreign_table_options(self, schema: str) -> List[Tuple[str, Dict[str, Any]]]:
+        """
+        Get a list of options the foreign tables in this schema were instantiated with
+        :return: List of tables and their options
+        """
+        # We use this to suggest table options during introspection. With FDWs, we do this by
+        # mounting the data source first (using IMPORT FOREIGN SCHEMA) and then scraping the
+        # foreign table options it inferred.
+
+        # Downstream FDWs can override this: if they remap some table options into different
+        # FDW options (e.g. "remote_schema" on the data source side turns into "schema" on the
+        # FDW side), they have to map them back in this routine (otherwise the introspection will
+        # suggest "schema", which is wrong.
+
+        # This is also used for type remapping, since table params can only be strings on PG.
+        # TODO: this will lead to a bunch of serialization/deserialization code duplication
+        #   in data sources. One potential solution is, at least in Multicorn-backed wrappers
+        #   that we control, passing a JSON as a single table param instead.
+
+        return cast(
+            List[Tuple[str, Dict[str, str]]],
+            self.engine.run_sql(
+                "SELECT foreign_table_name, json_object_agg(option_name, option_value) "
+                "FROM information_schema.foreign_table_options "
+                "WHERE foreign_table_schema = %s GROUP BY foreign_table_name",
+                (schema,),
+            ),
+        )
+
+    def preview(self, tables: Optional[TableInfo]) -> PreviewResult:
         # Preview data in tables mounted by this FDW / data source
 
         # Local import here since this data source gets imported by the commandline entry point
@@ -197,16 +244,23 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
 
         tmp_schema = get_temporary_table_id()
         try:
-            self.mount(tmp_schema, tables=schema)
+            # Seed the result with the tables that failed to mount
+            result: PreviewResult = {
+                e.table_name: e for e in self.mount(tmp_schema, tables=tables) or []
+            }
+
             # Commit so that errors don't cancel the mount
             self.engine.commit()
-            result: Dict[str, Union[str, List[Dict[str, Any]]]] = {}
             for t in self.engine.get_all_tables(tmp_schema):
+                if t in result:
+                    continue
                 try:
                     result[t] = self._preview_table(tmp_schema, t)
                 except psycopg2.DatabaseError as e:
                     logging.warning("Could not preview data for table %s", t, exc_info=e)
-                    result[t] = str(e)
+                    result[t] = MountError(
+                        table_name=t, error=get_exception_name(e), error_text=str(e).strip()
+                    )
             return result
         finally:
             self.engine.rollback()
@@ -286,9 +340,14 @@ def _format_options(option_names):
     )
 
 
-def _import_foreign_schema(
-    engine: "PostgresEngine", mountpoint: str, remote_schema: str, server_id: str, tables: List[str]
-) -> None:
+def import_foreign_schema(
+    engine: "PostgresEngine",
+    mountpoint: str,
+    remote_schema: str,
+    server_id: str,
+    tables: List[str],
+    options: Optional[Dict[str, str]] = None,
+) -> List[MountError]:
     from psycopg2.sql import Identifier, SQL
 
     # Construct a query: import schema limit to (%s, %s, ...) from server mountpoint_server into mountpoint
@@ -296,7 +355,33 @@ def _import_foreign_schema(
     if tables:
         query += SQL("LIMIT TO (") + SQL(",").join(Identifier(t) for t in tables) + SQL(")")
     query += SQL("FROM SERVER {} INTO {}").format(Identifier(server_id), Identifier(mountpoint))
-    engine.run_sql(query)
+    args: List[str] = []
+
+    if options:
+        query += _format_options(options.keys())
+        args.extend(options.values())
+
+    engine.run_sql(query, args)
+
+    # Some of our FDWs use the PG notices as a side channel to pass errors and information back
+    # to the user, as IMPORT FOREIGN SCHEMA is all-or-nothing. If some of the tables failed to
+    # mount, we return those + the reasons.
+    import_errors: List[MountError] = []
+    if engine.notices:
+        for notice in engine.notices:
+            match = re.match(r".*SPLITGRAPH: (.*)\n", notice)
+            if not match:
+                continue
+            notice_j = json.loads(match.group(1))
+            import_errors.append(
+                MountError(
+                    table_name=notice_j["table_name"],
+                    error=notice_j["error"],
+                    error_text=notice_j["error_text"],
+                )
+            )
+
+    return import_errors
 
 
 def create_foreign_table(
@@ -324,7 +409,9 @@ def create_foreign_table(
     for col in schema_spec:
         if col.comment:
             query += SQL("COMMENT ON COLUMN {}.{}.{} IS %s;").format(
-                Identifier(schema), Identifier(table_name), Identifier(col.name),
+                Identifier(schema),
+                Identifier(table_name),
+                Identifier(col.name),
             )
             args.append(col.comment)
     return query, args
@@ -342,6 +429,12 @@ class PostgreSQLDataSource(ForeignDataWrapperDataSource):
             "based on postgres_fdw"
         )
 
+    credentials_schema = {
+        "type": "object",
+        "properties": {"username": {"type": "string"}, "password": {"type": "string"}},
+        "required": ["username", "password"],
+    }
+
     params_schema = {
         "type": "object",
         "properties": {
@@ -349,11 +442,12 @@ class PostgreSQLDataSource(ForeignDataWrapperDataSource):
             "port": {"type": "integer", "description": "Port"},
             "dbname": {"type": "string", "description": "Database name"},
             "remote_schema": {"type": "string", "description": "Remote schema name"},
-            "tables": _table_options_schema,
         },
         "required": ["host", "port", "dbname", "remote_schema"],
     }
 
+    table_params_schema = {"type": "object"}
+
     commandline_help: str = """Mount a Postgres database.
 
 Mounts a schema on a remote Postgres database as a set of foreign tables locally."""
@@ -378,8 +472,10 @@ If a dictionary, must have the format
     def get_user_options(self):
         return {"user": self.credentials["username"], "password": self.credentials["password"]}
 
-    def get_table_options(self, table_name: str):
-        return {"schema_name": self.params["remote_schema"]}
+    def get_table_options(self, table_name: str, tables: Optional[TableInfo] = None):
+        options = super().get_table_options(table_name, tables)
+        options["schema_name"] = self.params["remote_schema"]
+        return options
 
     def get_fdw_name(self):
         return "postgres_fdw"
@@ -389,27 +485,28 @@ If a dictionary, must have the format
 
 
 class MongoDataSource(ForeignDataWrapperDataSource):
+    credentials_schema = {
+        "type": "object",
+        "properties": {"username": {"type": "string"}, "password": {"type": "string"}},
+        "required": ["username", "password"],
+    }
+
     params_schema = {
         "type": "object",
         "properties": {
             "host": {"type": "string"},
             "port": {"type": "integer"},
-            "tables": {
-                "type": "object",
-                "additionalProperties": {
-                    "options": {
-                        "type": "object",
-                        "properties": {
-                            "db": {"type": "string"},
-                            "coll": {"type": "string"},
-                            "required": ["db", "coll"],
-                        },
-                    },
-                    "required": ["options"],
-                },
-            },
         },
-        "required": ["host", "port", "tables"],
+        "required": ["host", "port"],
+    }
+
+    table_params_schema = {
+        "type": "object",
+        "properties": {
+            "database": {"type": "string"},
+            "collection": {"type": "string"},
+        },
+        "required": ["database", "collection"],
     }
 
     commandline_help = """Mount a Mongo database.
@@ -421,7 +518,7 @@ Mounts one or more collections on a remote Mongo database as a set of foreign ta
 {
     "table_name": {
         "schema": {"col1": "type1"...},
-        "options": {"db": <dbname>, "coll": <collection>} 
+        "options": {"database": <dbname>, "collection": <collection>} 
     } 
 }
 ```
@@ -444,14 +541,6 @@ Mounts one or more collections on a remote Mongo database as a set of foreign ta
     def get_user_options(self):
         return {"username": self.credentials["username"], "password": self.credentials["password"]}
 
-    def get_table_options(self, table_name: str):
-        try:
-            table_params = self.params["tables"][table_name]["options"]
-        except KeyError:
-            raise ValueError("No options specified for table %s!" % table_name)
-
-        return {"database": table_params["db"], "collection": table_params["coll"]}
-
     def get_fdw_name(self):
         return "mongo_fdw"
 
@@ -464,22 +553,27 @@ Mounts one or more collections on a remote Mongo database as a set of foreign ta
 
 
 class MySQLDataSource(ForeignDataWrapperDataSource):
+    credentials_schema = {
+        "type": "object",
+        "properties": {"username": {"type": "string"}, "password": {"type": "string"}},
+        "required": ["username", "password"],
+    }
+
     params_schema = {
         "type": "object",
         "properties": {
             "host": {"type": "string"},
             "port": {"type": "integer"},
-            "remote_schema": {"type": "string"},
-            "tables": _table_options_schema,
+            "dbname": {"type": "string"},
         },
-        "required": ["host", "port", "remote_schema"],
+        "required": ["host", "port", "dbname"],
     }
 
     commandline_help: str = """Mount a MySQL database.
 
 Mounts a schema on a remote MySQL database as a set of foreign tables locally."""
 
-    commandline_kwargs_help: str = """remote_schema: Remote schema name (required)
+    commandline_kwargs_help: str = """dbname: Remote MySQL database name (required)
 tables: Tables to mount (default all). If a list, then will use IMPORT FOREIGN SCHEMA.
 If a dictionary, must have the format
     {"table_name": {"schema": {"col_1": "type_1", ...},
@@ -492,7 +586,7 @@ If a dictionary, must have the format
 
     @classmethod
     def get_description(cls) -> str:
-        return "Data source for MySQL databases that supports live querying, " "based on mysql_fdw"
+        return "Data source for MySQL databases that supports live querying, based on mysql_fdw"
 
     def get_server_options(self):
         return {
@@ -504,14 +598,16 @@ If a dictionary, must have the format
     def get_user_options(self):
         return {"username": self.credentials["username"], "password": self.credentials["password"]}
 
-    def get_table_options(self, table_name: str):
-        return {"dbname": self.params["remote_schema"]}
+    def get_table_options(self, table_name: str, tables: Optional[TableInfo] = None):
+        options = super().get_table_options(table_name, tables)
+        options["dbname"] = options.get("dbname", self.params["dbname"])
+        return options
 
     def get_fdw_name(self):
         return "mysql_fdw"
 
     def get_remote_schema_name(self) -> str:
-        return str(self.params["remote_schema"])
+        return str(self.params["dbname"])
 
 
 class ElasticSearchDataSource(ForeignDataWrapperDataSource):
@@ -528,42 +624,39 @@ class ElasticSearchDataSource(ForeignDataWrapperDataSource):
         "properties": {
             "host": {"type": "string"},
             "port": {"type": "integer"},
-            "tables": {
-                "type": "object",
-                "additionalProperties": {
-                    "options": {
-                        "type": "object",
-                        "properties": {
-                            "index": {
-                                "type": "string",
-                                "description": 'ES index name or pattern to use, for example, "events-*"',
-                            },
-                            "type": {
-                                "type": "string",
-                                "description": "Pre-ES7 doc_type, not required in ES7 or later",
-                            },
-                            "query_column": {
-                                "type": "string",
-                                "description": "Name of the column to use to pass queries in",
-                            },
-                            "score_column": {
-                                "type": "string",
-                                "description": "Name of the column with the document score",
-                            },
-                            "scroll_size": {
-                                "type": "integer",
-                                "description": "Fetch size, default 1000",
-                            },
-                            "scroll_duration": {
-                                "type": "string",
-                                "description": "How long to hold the scroll context open for, default 10m",
-                            },
-                        },
-                    },
-                },
+        },
+        "required": ["host", "port"],
+    }
+
+    table_params_schema = {
+        "type": "object",
+        "properties": {
+            "index": {
+                "type": "string",
+                "description": 'ES index name or pattern to use, for example, "events-*"',
+            },
+            "type": {
+                "type": "string",
+                "description": "Pre-ES7 doc_type, not required in ES7 or later",
+            },
+            "query_column": {
+                "type": "string",
+                "description": "Name of the column to use to pass queries in",
+            },
+            "score_column": {
+                "type": "string",
+                "description": "Name of the column with the document score",
+            },
+            "scroll_size": {
+                "type": "integer",
+                "description": "Fetch size, default 1000",
+            },
+            "scroll_duration": {
+                "type": "string",
+                "description": "How long to hold the scroll context open for, default 10m",
             },
         },
-        "required": ["host", "port", "tables"],
+        "required": ["index"],
     }
 
     commandline_help = """Mount an ElasticSearch instance.
